unit Unit1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ExtCtrls;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    CBSeries: TComboBox;
    CBSeriesActive: TCheckBox;
    CBOtherSide: TCheckBox;
    CBAxisTitle: TCheckBox;
    BToggleTheme: TButton;
    procedure FormCreate(Sender: TObject);
    procedure CBSeriesChange(Sender: TObject);
    procedure CBSeriesActiveClick(Sender: TObject);
    procedure CBOtherSideClick(Sender: TObject);
    procedure CBAxisTitleClick(Sender: TObject);
    procedure BToggleThemeClick(Sender: TObject);
  private
    { Private declarations }
    procedure ChartBeforeDrawChart(Sender: TObject);
    procedure PlaceAxes(nSeries: Integer=0; NextXLeft: Integer=0; NextXRight: Integer=0; MargLeft: Integer=0; MargRight: Integer=0);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses Chart, Series, TeeProcs, TeEngine, TeCanvas, TeeScrollPagerTool;

type TCustomTeePanelAccess=class(TCustomTeePanel);

var Chart1: TChart;
    ScrollPagerTool: TScrollPagerTool;

const
  darkGray=$2f2f2f;
  lightGray=$a9a9a9;

procedure TForm1.FormCreate(Sender: TObject);
var
  i: Integer;
const
  nSeries=4;
begin
  Chart1:=TChart.Create(Self);

  with Chart1 do
  begin
    Parent:=Self;
    Align:=alClient;
    Color:=darkGray;
    Gradient.Visible:=False;
    Walls.Hide;
    Legend.Hide;
    View3D:=False;

    Axes.Bottom.Axis.Color:=lightGray;
    Axes.Bottom.LabelsFont.Color:=lightGray;
    Axes.Bottom.Ticks.Color:=lightGray;
    Axes.Bottom.MinorTicks.Hide;
    Axes.Bottom.Grid.Hide;
  end;

  ScrollPagerTool:=TScrollPagerTool(Chart1.Tools.Add(TScrollPagerTool));
  with ScrollPagerTool do
  begin
    Align:=saCustom;
    DivisionRatio:=4;
    SubChartTChart.Walls.Hide;

    with SubChartTChart.Axes.Bottom do
    begin
      Axis.Color:=lightGray;
      LabelsFont.Color:=lightGray;
      Ticks.Color:=lightGray;
      MinorTicks.Hide;
      Grid.Hide;
    end;

    with SubChartTChart.Axes.Left do
    begin
      Axis.Color:=lightGray;
      LabelsFont.Color:=lightGray;
      Ticks.Color:=lightGray;
      MinorTicks.Hide;
      Grid.Hide;
    end;
  end;

  for i:=0 to nSeries-1 do
  begin
    Chart1.AddSeries(TLineSeries);

    if i=0 then
       ScrollPagerTool.AddSeries(Chart1[i]);

    Chart1.CustomAxes.Add;
    Chart1[i].CustomVertAxis:=Chart1.CustomAxes.Items[i];

    with Chart1.CustomAxes.Items[i] do
    begin
      Axis.Color:=Chart1[i].Color;
      LabelsFont.Color:=lightGray;
      Ticks.Color:=lightGray;
      MinorTicks.Hide;
      Grid.Visible:=False;
      Title.Text:='Axis'+IntToStr(i+1);
      Title.Font.Color:=Chart1[i].Color;
    end;

    CBSeries.Items.Add('Series'+IntToStr(i+1));
    Chart1[i].FillSampleValues(200);
    Chart1.CustomAxes.Items[i].PositionUnits:=muPixels;
    if ((i+1) mod 2 = 0) then
      Chart1.CustomAxes.Items[i].OtherSide:=True;
  end;

  Chart1.MarginUnits:=muPixels;
  Chart1.MarginTop:=100;

  CBSeries.ItemIndex:=0;
  CBSeriesActive.Checked:=True;
  CBOtherSide.Checked:=Chart1.CustomAxes.Items[0].OtherSide;
  CBAxisTitle.Checked:=Chart1.CustomAxes.Items[0].Title.Visible;

  Chart1.OnBeforeDrawChart:=ChartBeforeDrawChart;
  Chart1.Draw;
end;

procedure TForm1.ChartBeforeDrawChart(Sender: TObject);
begin
  PlaceAxes;
  TCustomTeePanelAccess(Chart1).ApplyMargins;
end;

procedure TForm1.PlaceAxes(nSeries: Integer=0; NextXLeft: Integer=0; NextXRight: Integer=0; MargLeft: Integer=0; MargRight: Integer=0);
const extraPos=12;
var tmpMargin: Integer;
begin
  if Chart1[nSeries].Active then
  begin
    if Chart1.CustomAxes.Items[nSeries].Title.Visible then
       tmpMargin:=Abs(Chart1.CustomAxes.Items[nSeries].Title.Height)
    else
       tmpMargin:=0;

    Inc(tmpMargin,Chart1.CustomAxes.Items[nSeries].MaxLabelsWidth);
    Inc(tmpMargin,Chart1.CustomAxes.Items[nSeries].TickLength);
    Inc(tmpMargin,extraPos);

    if Chart1.CustomAxes.Items[nSeries].OtherSide then
    begin
      Chart1.CustomAxes.Items[nSeries].PositionPercent:=NextXRight;
      NextXRight:=NextXRight-tmpMargin;
      MargRight:=MargRight+tmpMargin;
    end
    else
    begin
      Chart1.CustomAxes.Items[nSeries].PositionPercent:=NextXLeft;
      NextXLeft:=NextXLeft-tmpMargin;
      MargLeft:=MargLeft+tmpMargin;
    end;
  end;

  Chart1.MarginLeft:=MargLeft;
  Chart1.MarginRight:=MargRight;

  with ScrollPagerTool.SubChartTool.Charts[0] do
  begin
    Top:=0;
    Left:=0;
    Width:=Chart1.Width;
    Height:=Round(Chart1.Height / ScrollPagerTool.DivisionRatio);

    Chart.MarginUnits:=Chart1.MarginUnits;
    Chart.MarginLeft:=MargLeft-Chart.Axes.Left.MaxLabelsWidth-Chart.Axes.Left.TickLength;
    Chart.MarginRight:=Chart1.MarginRight;
  end;

  Inc(nSeries);

  if nSeries <= Chart1.SeriesCount-1 then
     PlaceAxes(nSeries, NextXLeft, NextXRight, MargLeft, MargRight);
end;

procedure TForm1.CBSeriesActiveClick(Sender: TObject);
begin
  Chart1[CBSeries.ItemIndex].Active:=CBSeriesActive.Checked;
  Chart1.CustomAxes[CBSeries.ItemIndex].Visible:=CBSeriesActive.Checked;
end;

procedure TForm1.BToggleThemeClick(Sender: TObject);
var
  i: Integer;
  priColor: TColor;
  secColor: TColor;
begin
  if BToggleTheme.Caption='Light' then
  begin
    priColor:=clWhite;
    secColor:=clBlack;
    BToggleTheme.Caption:='Dark';
  end
  else
  begin
    priColor:=darkGray;
    secColor:=lightGray;
    BToggleTheme.Caption:='Light';
  end;

  with Chart1 do
  begin
    Color:=priColor;

    for i:=0 to Chart1.CustomAxes.Count-1 do
      with Chart1.CustomAxes[i] do
      begin
        LabelsFont.Color:=secColor;
        Ticks.Color:=secColor;
      end;

    with Axes.Bottom do
    begin
      Axis.Color:=secColor;
      Axes.Bottom.LabelsFont.Color:=secColor;
      Axes.Bottom.Ticks.Color:=secColor;
    end;
  end;

  with ScrollPagerTool.SubChartTChart do
  begin
    with Axes.Left do
    begin
      Axis.Color:=secColor;
      LabelsFont.Color:=secColor;
      Ticks.Color:=secColor;
    end;

    with Axes.Bottom do
    begin
      Axis.Color:=secColor;
      LabelsFont.Color:=secColor;
      Ticks.Color:=secColor;
    end;
  end;
end;

procedure TForm1.CBAxisTitleClick(Sender: TObject);
begin
  Chart1.CustomAxes.Items[CBSeries.ItemIndex].Title.Visible:=CBAxisTitle.Checked;
end;

procedure TForm1.CBOtherSideClick(Sender: TObject);
begin
  Chart1.CustomAxes.Items[CBSeries.ItemIndex].OtherSide:=CBOtherSide.Checked;
end;

procedure TForm1.CBSeriesChange(Sender: TObject);
begin
  CBSeriesActive.Checked:=Chart1[CBSeries.ItemIndex].Active;
  CBOtherSide.Checked:=Chart1.CustomAxes.Items[CBSeries.ItemIndex].OtherSide;
  CBAxisTitle.Checked:=Chart1.CustomAxes.Items[CBSeries.ItemIndex].Title.Visible;
end;

end.
